@@ -9,7 +9,7 @@ use Carp;
 
 our @ISA = qw();
 
-our $VERSION = '0.03';
+our $VERSION = '0.01';
 
 
 # Constructor
@@ -66,11 +66,6 @@ sub isa {
 sub children {
   my $self = shift;
 
-  if (@_) {
-    my $chldlst = shift;
-    $self->{'chldlst'} = $chldlst;
-  }
-
   return $self->{'chldlst'}
 }
 
@@ -79,11 +74,6 @@ sub children {
 sub element {
   my $self = shift;
 
-  if (@_) {
-    my $element = shift;
-    $self->{'eltname'} = $element;
-  }
-
   return $self->{'eltname'}
 }
 
@@ -92,11 +82,6 @@ sub element {
 sub combineop {
   my $self = shift;
 
-  if (@_) {
-    my $combnop = shift;
-    $self->{'combnop'} = $combnop;
-  }
-
   return $self->{'combnop'};
 }
 
@@ -105,11 +90,6 @@ sub combineop {
 sub occurop {
   my $self = shift;
 
-  if (@_) {
-    my $occurop = shift;
-    $self->{'occurop'} = $occurop;
-  }
-
   return $self->{'occurop'};
 }
 
@@ -148,12 +128,19 @@ sub childnames {
 sub string {
   my $self = shift;
 
-  my $type = $self->type;
-  if ($self->isatomic and ($type eq 'mixed' or $type eq 'element')) {
-    return "(" . $self->_string . ")";
+  my $str = '';
+  if ($self->isatomic) {
+    $str = $self->element;
   } else {
-    return $self->_string;
+    my $strlst = [];
+    my $child;
+    foreach $child ( @{$self->{'chldlst'}} ) {
+      push @$strlst, $child->string;
+    }
+    $str .= '(' . join($self->combineop,@$strlst) . ')';
   }
+  $str .= $self->occurop if (defined $self->occurop);
+  return $str;
 }
 
 
@@ -226,8 +213,7 @@ sub type {
     }
   } else {
     my $oop = (defined $self->occurop)?$self->occurop:'';
-    my $cop = (defined $self->combineop)?$self->combineop:'';
-    if ($cop eq '|' and ($oop eq '' or $oop eq '*')) {
+    if ($self->combineop eq '|' and ($oop eq '' or $oop eq '*')) {
       my $chld = $self->children;
       my $c;
       foreach $c (@$chld) {
@@ -273,7 +259,7 @@ sub dfa {
   # Remove unreachable states
   $fsa->rmunreach;
   # Ensure FSA is a DFA
-  carp "FSA is not deterministic\n" if (!$fsa->isdeterministic);
+  warn "FSA is not deterministic\n" if (!$fsa->isdeterministic);
   return $fsa;
 }
 
@@ -291,13 +277,8 @@ sub _parse {
   # Substitute entity values for references
   if (defined $entmn and
       $cmstr =~ /^%([\w\.:\-_]+);$|^\(%([\w\.:\-_]+);\)$/) {
-    my $paren = defined $2;
-    $self->{'peref'} = $paren ? $2 : $1;
-    my $entv = $entmn->pevalue($self->{'peref'});
-    if (defined $entv) {
-      $cmstr = $entv;
-      $cmstr = '(' . $cmstr . ')' if ($paren);
-    }
+    $self->{'peref'} = defined($1)?$1:$2;
+    $cmstr = $entmn->pevalue($self->{'peref'});
   }
 
   # Temporary
@@ -330,15 +311,15 @@ sub _parse {
     } else { # Parenthesis first
       my ($mat, $pst) = _parenmatch($cmstr);
       # Check whether parenthesis post-match consists of an optional
-      # occurence operator optionally followed by a combine operator
-      if ($pst =~ /^(\?|\+|\*)?(\,|\|)?/) {
+      # occurence operator followed by a combine operator
+      if ($pst =~ /^(\?|\+|\*)?(\,|\|)/) {
 	$expr = $mat.(defined($1)?$1:'');
 	$self->{'combnop'} = $2;
 	#print "CMBNOP: $2   $cmstr\n";
 	$cmstr = $';
 	push @{$self->{'chldlst'}}, $class->new($expr, $entmn);
       } else {
-	carp "invalid content model: $cmstr\n";
+	warn "invalid content model: $cmstr\n";
 	return;
       }
     }
@@ -363,13 +344,13 @@ sub _parse {
 	  $cmstr = $';
 	  push @{$self->{'chldlst'}}, $class->new($expr, $entmn);
 	} else {
-	  carp "invalid content model: $cmstr\n";
+	  warn "invalid content model: $cmstr\n";
 	  return;
 	}
       }
     }
   } else {
-    carp "invalid content model: $cmstr\n";
+    warn "invalid content model: $cmstr\n";
     return;
   }
 }
@@ -381,64 +362,53 @@ sub _parse {
 sub _parenmatch {
   my $str = shift;
 
-  my $level = 0;
-  my $pos = 0;
-  my $len = length $str;
-  my $posl = index $str, '('; $posl = $len if ($posl < 0);
-  my $posr = index $str, ')'; $posr = $len if ($posr < 0);
-  if ($posl >= $len && $posr >= $len) {
-    # String contains no parentheses
+  if ($str =~ /^([^\(\)]*)$/) { # String contains no parentheses
     return ('',$str);
-  }
-  do {
-    if ($posl < $posr) {
-      # A ( is next
-      $level++;
-      $pos = $posl+1;
-      $posl = index $str, '(', $pos;
-      $posl = $len if ($posl < 0);
-    } else { # $posl >= $posr
-      # a ) is next
-      $level--;
-      if ($level < 0) {
-	carp "Parenthesis matching error in $str\n";
-	return undef;
-      }
-      $pos = $posr+1;
-      $posr = index $str, ')', $pos;
-      $posr = $len if ($posr < 0);
+  } elsif ($str =~ /^([^\(\)]*\()/) { # String contains an opening parenthesis
+    # Set pre-match string to substring up to opening parenthesis
+    my $pre = $1;
+    # Call recursive part of function on substring after opening parenthesis
+    my ($m, $p) = _parenrecurse($');
+    if (defined $m) {
+      return ($pre.$m, $p);
+    } else {
+      return undef;
     }
-    # Drop out when the level returns to 0 or the string is exhausted
-  } while ($level > 0 && $pos < $len);
-  if ($level > 0) {
-    carp "Parenthesis matching error in $str\n";
+  } else {
+    warn "Parenthesis matching error in $str\n";
     return undef;
   }
-  my $pre = substr $str, 0, $pos;
-  my $pst = substr $str, $pos;
-  return ($pre, $pst);
 }
 
 
-# Recursive part of function to build a string representation of the
-# content model
-sub _string {
-  my $self = shift;
+# Recursive part of function for parenthesis matching
+sub _parenrecurse {
+  my $str = shift;
 
-  my $str = '';
-  if ($self->isatomic) {
-    $str = $self->element;
+  # Initialise match string
+  my $mat = '';
+  # Initialise post-match string
+  my $pst = $str;
+  if ($str =~ /^([^\(\)]*\()/) { # String contains an opening parenthesis
+    # Set pre-match string to substring up to opening parenthesis
+    my $pre = $1;
+    # Do recursive call on substring after opening parenthesis
+    my ($m, $p) = _parenrecurse($');
+    # Append pre-match string and new matching component to match string
+    $mat .= $pre.$m;
+    # Set post-match string to new post-match component
+    $pst = $p;
+  }
+  if ($pst =~ /^([^\(\)]*\))/) { # Post-match contains a closing parenthesis
+    # Append substring up to closing parenthesis to match string
+    $mat .= $1;
+    # Set post-match to subtrstring after closing parenthesis
+    $pst = $';
+    return ($mat, $pst);
   } else {
-    my $strlst = [];
-    my $child;
-    foreach $child ( @{$self->{'chldlst'}} ) {
-      push @$strlst, $child->_string;
-    }
-    my $cop = (defined $self->combineop)?$self->combineop:'';
-    $str .= '(' . join($cop,@$strlst) . ')';
+    warn "Parenthesis matching error in $str\n";
+    return undef;
   }
-  $str .= $self->occurop if (defined $self->occurop);
-  return $str;
 }
 
 
@@ -569,42 +539,23 @@ Test object type.
 
 Return the list of child objects (subexpressions).
 
- my $objlst = $cm->children($children);
-
-Sets the list of child objects (subexpressions). Returns the new value.
-
 =item B<element>
 
  my $name = $cm->element;
 
 Return the element name if the object has no subexpressions.
 
- my $name = $cm->element($elename);
-
-Set the element name. The element name should only be set
-if the object has no subexpressions. Returns the new value.
-
 =item B<combineop>
 
  my $op = $cm->combineop;
 
-Return the combination operator (",", "|" or C<undef>).
-
- my $op = $cm->combineop($combineop);
-
-Set the combination operator (",", "|", or C<undef>).
-Returns the new value.
+Return the combination operator ("," or "|").
 
 =item B<occurop>
 
  my $op = $cm->occurop;
 
-Return the occurrence operator ("?", "+", "*", or C<undef>).
-
- my $op = $cm->occurop($occurop);
-
-Set the occurrence operator ("?", "+", "*", or C<undef>).
-Returns the new value.
+Return the occurrence operator ("?","+", or "*").
 
 =item B<isatomic>
 
@@ -666,15 +617,9 @@ Brendt Wohlberg E<lt>wohl@cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2006-2007 by Brendt Wohlberg
+Copyright (C) 2006 by Brendt Wohlberg
 
 This library is available under the terms of the GNU General Public
 License (GPL), described in the GPL file included in this distribution.
 
-=head1 ACKNOWLEDGMENTS
-
-Peter Lamb E<lt>Peter.Lamb@csiro.auE<gt> fixed a bug in the _parse
-function, provided an improved implementation of _parenmatch, and
-modified accessor methods to allow setting of relevant values.
-
 =cut
