@@ -10,7 +10,7 @@ use Carp;
 
 our @ISA = qw(XML::DTD::Component);
 
-our $VERSION = '0.02';
+our $VERSION = '0.01';
 
 
 # Constructor
@@ -93,12 +93,9 @@ sub _parse {
     $self->{'WS1'} = $3;
     my $cntspc = $4;
     $self->{'WS2'} = $5;
-
-    $name = $self->_entitymanager->peexpend($name)
-      if ($name =~ /^%([\w\.:\-_]+);$/);
-
+    # Still need to handle name being a peref
     $self->{'NAME'} = $name;
-
+    #$cntspc = $entman->entitysubst($cntspc);
     $self->{'CONTENTSPECTEXT'} = $cntspc;
     $self->{'CONTENTSPEC'} = XML::DTD::ContentModel->new($cntspc, $entman);
   } else {
@@ -178,8 +175,4 @@ Copyright (C) 2004-2006 by Brendt Wohlberg
 This library is available under the terms of the GNU General Public
 License (GPL), described in the GPL file included in this distribution.
 
-=head1 ACKNOWLEDGMENTS
-
-Peter Lamb E<lt>Peter.Lamb@csiro.auE<gt> improved entity substitution.
-
 =cut
